import { instrumentCron } from './cron.js';
import { instrumentNodeCron } from './node-cron.js';
import { instrumentNodeSchedule } from './node-schedule.js';

/** Methods to instrument cron libraries for Sentry check-ins */
const cron = {
  instrumentCron,
  instrumentNodeCron,
  instrumentNodeSchedule,
};

export { cron };
