Object.defineProperty(exports, '__esModule', { value: true });

const core = require('@sentry/core');

/** Adds an origin to an OTEL Span. */
function addOriginToSpan(span, origin) {
  span.setAttribute(core.SEMANTIC_ATTRIBUTE_SENTRY_ORIGIN, origin);
}

exports.addOriginToSpan = addOriginToSpan;
